/*
 * main.h
 * 20110422 (c) DE0508, Uwe
 */

#ifndef _INCL_MAIN_H
#define _INCL_MAIN_H

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>

/*
 * Typedefs und Defines
 */
#define nop()	__asm__ __volatile__ ("nop")

#define TRUE  1
#define FALSE 0
typedef unsigned char bool_t;


// wir definieren die Timer0 Werte
#define T0_PRESCALE	256
#define T0_COUNTER	(uint8_t)((double)F_CPU / T0_PRESCALE / 40U +0.5)


/*
 * PROGMEM Textstrings
 */

#ifndef PROGMEM_DATA
#define PROGMEM_DATA(_typ_, _var_)	\
extern const _typ_ _var_ PROGMEM;	\
const _typ_ _var_

#define PROGMEM_DATA_PROTO(_typ_, _var_) \
extern const _typ_ _var_ PROGMEM;
#endif

/*
 * Ein- und Ausgabe Macros
 */

#define sbit(_port, _bit)		_port |= _BV(_bit)	// set bit
#define cbit(_port, _bit)		_port &= ~_BV(_bit)	// clear bit
#define tbit(_port, _bit)		_port ^= _BV(_bit)	// toggle bit

/*
 * Bit-Macros
 */
#ifndef SBIT
struct bits {
  uint8_t b0:1;
  uint8_t b1:1;
  uint8_t b2:1;
  uint8_t b3:1;
  uint8_t b4:1;
  uint8_t b5:1;
  uint8_t b6:1;
  uint8_t b7:1;
} __attribute__((__packed__));

#define SBIT_(port,pin) ((*(volatile struct bits*)&port).b##pin)
#define	SBIT(x,y)	SBIT_(x,y)
#endif

/*
 * Lowlevel Routinen
 */
#define SET_OUTPUT_(_ddr,_pin)		_ddr |= _BV(_pin)
#define SET_OUTPUT_MASK(_ddr,_mask)	_ddr |= (_mask)

#define SET_INPUT_(_ddr,_pin)		_ddr &= ~_BV(_pin)
#define SET_INPUT_MASK_(_ddr,_mask)	_ddr &= ~(_mask)

#define SET_PORT_ON_(_port,_pin)	_port |= _BV(_pin)
#define SET_PORT_ON_MASK_(_port,_mask)	_port |= (_mask)

#define SET_PORT_OFF_(_port,_pin)	_port &= ~_BV(_pin)
#define SET_PORT_OFF_MASK_(_port,_mask)	_port &= ~(_mask)

#define SET_TOGGLE_(_port,_pin)		_port ^= _BV(_pin)

#define SET_ADC_(_ddr,_pin)		_ddr &= ~_BV(_pin)

/*
 * Basis Routinen
 */

#define SET_OUTPUT(id)			SET_OUTPUT_(id##_##DDR, id##_##BIT)
#define SET_INPUT(id)			SET_INPUT_(id##_##DDR, id##_##BIT)
#define SET_INPUT_MASK(id)		SET_INPUT_MASK_(id##_##DDR, id##_##BITS)
#define SET_ADC(id)			SET_ADC_(id##_##DDR, id##_##BIT); SET_PORT_OFF_(id##_##PORT, id##_##BIT)

#define SET_PULLUP_ON(id)		SET_PORT_ON_(id##_##PORT, id##_##BIT)
#define SET_PULLUP_ON_MASK(id)		SET_PORT_ON_MASK_(id##_##PORT, id##_##BITS)
#define SET_PULLUP_OFF(id)		SET_PORT_OFF_(id##_##PORT, id##_##BIT)
#define SET_PULLUP_OFF_MASK(id)		SET_PORT_OFF_MASK_(id##_##PORT, id##_##BITS)
#define SET_ON(id)			SET_PORT_ON_(id##_##PORT, id##_##BIT)
#define SET_OFF(id)			SET_PORT_OFF_(id##_##PORT, id##_##BIT)
#define SET_TOGGLE(id)			SET_TOGGLE_(id##_##PORT, id##_##BIT)

#define TEST_INPUT_(_pin , _bit)	((_pin) & _BV(_bit))
#define TEST_INPUT(id)			TEST_INPUT_(id##_##PIN, id##_##BIT)

/*
 * Ein- und Ausgabe Definitionen
 */

#define RELAIS1_DDR		DDRB
#define RELAIS1_PORT		PORTB
#define RELAIS1_BIT		PB0

#define RELAIS2_DDR		DDRB
#define RELAIS2_PORT		PORTB
#define RELAIS2_BIT		PB1

#define RELAIS3_DDR		DDRB
#define RELAIS3_PORT		PORTB
#define RELAIS3_BIT		PB2

#define LED_DDR			DDRB
#define LED_PORT		PORTB
#define LED_BIT			PB4

#define KEY_DDR			DDRB
#define KEY_PORT		PORTB
#define KEY_PIN			PINB
#define KEY_BIT			PB3

/*
 * Schduler
 */
#define USE_TASK_KEY
#define USE_TASK_LED

#define ADD_TASK_RELAIS1()		timeradd( task_relais1,	SECONDS(35) )
#define REM_TASK_RELAIS1()		timerremove( task_relais1 )
#define ADD_TASK_RELAIS2()		timeradd( task_relais2,	SECONDS(80) )
#define REM_TASK_RELAIS2()		timerremove( task_relais2 )
#define ADD_TASK_RELAIS3()		timeradd( task_relais3,	SECONDS(150) )
#define REM_TASK_RELAIS3()		timerremove( task_relais3 )

#define ADD_TASK_LED()			timeradd( task_led,	SECONDS(0.5) )
#define REM_TASK_LED()			timerremove( task_led )

#define ADD_TASK_KEY()			timeradd( task_key,	SECONDS(0.1) )
#define REM_TASK_KEY()			timerremove( task_key )


#ifdef GPIOR0
# define f_timer_tick	SBIT(GPIOR0,0)
#endif

/*
 * Prototypen
 */

#endif
