/*
 * schedule.c
 * 20110422 (c) DE0508, Uwe
 * 20110302 kleine Änderungen
 */

#include "schedule.h"

#define MAXTIMERS	6		// Benötigt 5 Byte * MAXTIMERS
#define T_FREE		254		// > MAXTIMERS
#define T_LAST		255

typedef struct{
  uchar_t next;				// next in list or end mark
  utimer_t delta;			// delta to previous entry
  funcp func;
} t_ctrl_struct;


// extern
#ifndef GPIOR0
volatile uchar_t f_timer_tick;
#endif


// static
static t_ctrl_struct t_ctrl_lst[MAXTIMERS];
static utimer_t t_delay;			// count down until next service
static uchar_t t_first;				// point to first entry


void timertick(void)
{
  t_ctrl_struct *p;				// for faster access

  --t_delay;
  while( t_delay == 0 ){			// serve if delay = 0
    if( t_first == T_LAST )			// no function to serve
      break;
    p = &t_ctrl_lst[t_first];
    t_first = p->next;				// point to next
    p->next = T_FREE;				// mark free
    t_delay = t_ctrl_lst[t_first].delta;        // next delay delta
    p->func();				    	// execute function
  }
}


uchar_t timeradd( funcp func, utimer_t delay )
{
  uchar_t n;
  uchar_t i;					// index
  uchar_t ipre = 0;				// previous index
  utimer_t d1;					// last delta delay
  t_ctrl_struct *p;				// for faster access

  for( n = 0;; ){
    p = &t_ctrl_lst[n];
    if( p->next == T_FREE )
      break;
    n++;
    if( n == MAXTIMERS )
      return 1;                                 // error, list full
  }

  i = t_first;
  d1 = t_delay;
  while( i != T_LAST ){				// check until end
    if( d1 >= delay ){ 	  	      		// last >= new
      t_ctrl_lst[i].delta = d1 - delay;		// correct following entry
      break;
    }else{
      delay -= d1;				// remaining delay
      ipre = i;                         	// previous entry
      i = t_ctrl_lst[i].next;			// index of next entry
      d1 = t_ctrl_lst[i].delta; 		// next delay delta
    }
  }						// insert new entry

  p->next = i;					// following entry
  p->delta = delay;				// store remaining delay
  p->func = func;

  if( i == t_first ){ 				// insert at first
    t_first = n;
    t_delay = delay;
  }else{
    t_ctrl_lst[ipre].next = n;			// previous entry
  }
  return 0;					// successful
}


uchar_t timerremove( funcp func )
{
  uchar_t ipre = 0;				// previous index
  uchar_t irem = 0;				// index to be removed
  uchar_t ifol = t_first;				// following index
  t_ctrl_struct *p;				// for faster access

  do {
    if( ifol == T_LAST )
      return 1;					// not found
    ipre = irem;
    irem = ifol;
    p = &t_ctrl_lst[irem];
    ifol = p->next;                             // get next
  } while( p->func != func );                    // found it

  p->next = T_FREE;                             // mark it as free
  if( irem == t_first ){
    t_first = ifol;                           	// serve next entry
    t_delay += t_ctrl_lst[ifol].delta;		// correct current delta
  }else{
    t_ctrl_lst[ipre].next = ifol;              	// skip index
    if( ifol != T_LAST )                   	// correct following delta
      t_ctrl_lst[ifol].delta += p->delta;
  }
  return 0;					// successful
}


void timerinit(void)
{
  uchar_t i;

  for( i = MAXTIMERS; i; i-- )
    t_ctrl_lst[i-1].next = T_FREE;	      	// mark all free
  t_first = T_LAST;              		// set no timer served
}

