/************************************************************************/
/*                                                                      */
/*                      Debouncing 8 Keys                               */
/*                      Sampling 4 Times                                */
/*                      With Repeat Function                            */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                                                                      */
/************************************************************************/

#include <avr/interrupt.h>
#include <inttypes.h>

#include "entprellen.h"

volatile uint8_t key_state;		// debounced and inverted key state:
					// bit = 1: key pressed
volatile uint8_t key_press;		// key press detect
volatile uint8_t key_rpt;		// key long press and repeat

#ifdef USE_KEY_RELEASE
volatile uint8_t key_release;
#endif

#ifdef USE_DEBOUNCE
void task_key_debounce(void)			// call every 16ms
{
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;
  static uint8_t rpt;				// bei 10ms ist die Zahl 8-Bit gross

  uint8_t i;

  i = key_state ^ ~KEY_PIN;                     // key changed ?
  ct0 = ~( ct0 & i );                           // reset or count ct0
  ct1 = ct0 ^ (ct1 & i);                        // reset or count ct1
  i &= ct0 & ct1;                               // count until roll over ?
  key_state ^= i;                               // then toggle debounced state
  key_press |= key_state & i;                   // 0->1: key press detect

#ifdef USE_KEY_RELEASE
  key_release |= ~key_state & i;		// 1->0: key release detect
#endif

  if( (key_state & REPEAT_MASK) == 0 )          // check repeat function
     rpt = REPEAT_START;                        // start delay
  if( --rpt == 0 ) {
    rpt = REPEAT_NEXT;                          // repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
}
#endif


uint8_t get_key_press( uint8_t key_mask )
{
  cli();
  key_mask &= key_press;                      // read key(s)
  key_press ^= key_mask;                      // clear key(s)
  sei();
  return key_mask;
}

#ifdef USE_KEY_RELEASE
uint8_t get_key_release( uint8_t key_mask )
{
  cli();
  key_mask &= key_release;   // read key(s)
  key_release ^= key_mask;   // clear key(s)
  sei();
  return key_mask;
}
#endif

uint8_t get_key_rpt( uint8_t key_mask )
{
  cli();
  key_mask &= key_rpt;                        // read key(s)
  key_rpt ^= key_mask;                        // clear key(s)
  sei();
  return key_mask;
}


uint8_t get_key_short( uint8_t key_mask )
{
  uint8_t i;

  cli();
  i = get_key_press( ~key_state & key_mask );
  sei();
  return i;
}


// ist als Macro definiert
#if 0
uint8_t get_key_long( uint8_t key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}


uint8_t get_key_long_r( uint8_t key_mask )      // if repeat function needed
{
  return get_key_press( get_key_rpt( key_press & key_mask ));
}


uint8_t get_key_rpt_l( uint8_t key_mask )       // if long function needed
{
  return get_key_rpt( ~key_press & key_mask );
}
#endif

