#ifndef _INCL_ENTPRELLEN_H
#define _INCL_ENTPRELLEN_H

#include <inttypes.h>
#include "main.h"
#include "timer.h"

#define USE_KEY_RELEASE
#undef USE_DEBOUNCE

/* Beispiele */
#if 0
#define KEY_PIN         PINB
#define KEY_PORT        PORTB
#define KEY0            PB0
#endif

#define REPEAT_MASK	(1<<KEY_BIT)				// repeat: key
#define REPEAT_START    (500/DEBOUNCETICK)			// after 500ms
#define REPEAT_NEXT     (200/DEBOUNCETICK)			// every 200ms

extern volatile uint8_t key_state;
extern volatile uint8_t key_press;
extern volatile uint8_t key_rpt;

#ifdef USE_KEY_RELEASE
extern volatile uint8_t key_release;
#endif

extern uint8_t get_key_press( uint8_t key_mask );

#ifdef USE_KEY_RELEASE
extern uint8_t get_key_release( uint8_t key_mask );
#endif

extern uint8_t get_key_rpt( uint8_t key_mask );
extern uint8_t get_key_short( uint8_t key_mask );

#define get_key_long(key_mask)		get_key_press( get_key_rpt( key_mask ))
#define get_key_long_r(key_mask)	get_key_press( get_key_rpt( key_press & key_mask ))
#define get_key_rpt_l(key_mask)		get_key_rpt( ~key_press & key_mask )


#ifdef USE_DEBOUNCE
extern void task_key_debounce(void);
#endif

/*
 * ISR_DEBOUNCE() wird in timer_wdt.c eingebunden
 */
#define ISR_DEBOUNCE()			{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  uint8_t i;					\
						\
  i = key_state ^ ~KEY_PIN;			\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
						\
  key_release |= ~key_state & i;		\
}

#define ISR_DEBOUNCE_REPEAT()			{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  static uint16_t rpt;				\
  uint8_t i;					\
						\
  i = key_state ^ ~KEY_PIN;			\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
						\
  key_release |= ~key_state & i;		\
						\
  if( (key_state & REPEAT_MASK) == 0 )		\
     rpt = REPEAT_START;			\
  if( --rpt == 0 ) {				\
    rpt = REPEAT_NEXT;				\
    key_rpt |= key_state & REPEAT_MASK;		\
  }						\
}

#endif
